package com.reptileclient.jlifeapp.config;

import com.github.xiaoymin.knife4j.spring.annotations.EnableKnife4j;
import com.google.common.base.Function;
import com.google.common.base.Optional;
import com.google.common.base.Predicate;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import springfox.documentation.RequestHandler;
import springfox.documentation.builders.ApiInfoBuilder;
import springfox.documentation.builders.ParameterBuilder;
import springfox.documentation.builders.PathSelectors;
import springfox.documentation.schema.ModelRef;
import springfox.documentation.service.ApiInfo;
import springfox.documentation.service.Parameter;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spring.web.plugins.Docket;
import springfox.documentation.swagger2.annotations.EnableSwagger2;

import java.util.ArrayList;
import java.util.List;

@Controller
@Configuration
@EnableSwagger2
@EnableKnife4j
public class SwaggerConfig {
	 
    //是否开启swagger，正式环境一般是需要关闭的，可根据springboot的多环境配置进行设置
//    @Value(value = "${swagger.enabled}")
    Boolean swaggerEnabled;
 
    @GetMapping("/")
    public String index() {
    	return "redirect:swagger-ui.html";
    }

    @Bean
    public Docket createRestApi() {
    	
    	ParameterBuilder token = new ParameterBuilder();
        token.name("token").description("user token")
    	.modelRef(new ModelRef("string")).parameterType("header") 
    	.required(false).build(); //header中的ticket参数非必填，传空也可以
        
        List<Parameter> pars = new ArrayList<Parameter>();  
    	pars.add(token.build()); 
    	
        return new Docket(DocumentationType.SWAGGER_2)
                .apiInfo(apiInfo())
                .select()
                .apis(basePackage("com.reptileclient.jlifeapp.web;com.reptileclient.jlifeapp.api;com.reptileclient.jlifeapp.restservice"))
                .paths(PathSelectors.any())
                .build()
                .globalOperationParameters(pars);
    }
    
    public static Predicate<RequestHandler> basePackage(final String basePackage) {
        return input -> declaringClass(input).transform(handlerPackage(basePackage)).or(true);
    }
    
    private static Function<Class<?>, Boolean> handlerPackage(final String basePackage)     {
        return input -> {
            // 循环判断匹配
            for (String strPackage : basePackage.split(";")) {
                boolean isMatch = input.getPackage().getName().startsWith(strPackage);
                if (isMatch) {
                    return true;
                }
            }
            return false;
        };
    }

    
    private static Optional<? extends Class<?>> declaringClass(RequestHandler input) {
        return Optional.fromNullable(input.declaringClass());
    }
    private ApiInfo apiInfo() {
        return new ApiInfoBuilder()
                .title("鲸致生活盒马订单api文档")
                .description("测试专用")
                .termsOfServiceUrl("http://192.168.1.173:1890/")
                .version("1.0")
                .build();
    } 
	
}

